% KADMOS: a Finite Element code for the calculation of apparent K-Ar ages in minerals
% Evangelos Moulas and Mark T Brandon
% doi: 10.5281/zenodo.7358138
% 
% code written by Evangelos Moulas, 30/09/2022
% Metamorphic Processes Research Group
% Johannes Gutenberg University, Mainz, Germany
%========================================================================
clear,clc,close all
%Problem Set UP =========================================================
tic
Inputs_Path_OCTAVE;
%Flags ==================================================================
flag_save   = 0;                 %Indicate if you want to save
flag_plot   = 1;                 %Indicate if you want to plot
flag_echo   = 1;                 %Indicate if you want to echo the results
%Check Number of Domains -------------------------------------------------
[ndd]                   = check_ndomains(Pres_a,TempK_a,time_a,Lr,ndim_a,Age0,Min_Sys);
%Numerical Parameters for FEM solution ==================================
chol_sol                =         1;           %Cholesky Solver
theta                   =         1;           %Crank-Nicolson Parameter
nt                      =       100;           %Time increments
nels                    =       100;           %Number of Elements
RdX                     =         2;           %Ratio of dX (1 for regular grid)
%plot options --------------------------------------------------------
nout                    =         2;           %Plot every nout iterations
LWW                     =       1.2;           %Linewidth of lines (in plot)
FSS                     =        12;           %Fontsize (in plot)
%========================================================================
%============== PLEASE DO NOT MODIFY BEYOND THIS POINT ==================
%Preprocessing & FEM ----------------------------------------------------
dt                      =   tlim/nt;           %Time discretization
nr                      =    nels+1;           %nx nodes in r-direction
[r1d,dr_ar,Cfact]       = linspace_adapt(0,1,nr,RdX);
%------------------------------------------------------------------------
el2n(1,:)    =    1:1:nr-1; %Extract nodes of element
el2n(2,:)    =    2:1:nr;   %nodes = el2n(:,elem);
%DOF-Node relation-------------------------------------------------------
el2d         =    el2n;
ndofs        =    nr*1;
%Usefull Constants ======================================================
c2K         =            273.15; %celcius 2 Kelvin
Myr2sec     =  3600*24*365.25e6; %Myr in secs
mic2met     =              1e-6; %from microns to cm
cm2metr     =              1e-2; %from cm to metre
micr2cm     =   mic2met/cm2metr; %from microns to cm
Rgas        =         8.3144621; %gas constant
TsurfK      = 25+c2K;            %Tsurface
TminK       = Tmin + c2K;        %MinT of Interpolation
f_loss      = 1.1247;            %Critical fractional loss value
%Argon decay----------------------------------------------------------
lamti       = 5.543*1e10*1e-6;   %Inverse tot Lamda in Myr
lam1i       = 0.581*1e10*1e-6;   %Inverse for lamda in Myr (no Ca)
%Preprocess for decay constants---------------------------------------
lamt        =        1./lamti;   %decay constants for Ar
lam1        =        1./lam1i;   %-//-
ke          =       lam1/lamt;   %ratio to total
%Initial Conditions --------------------------------------------------
rplot           =  repmat(r1d(:),1,ndd);         %Ndim Radial dist
Cloc            =  (1/ke).*(1-exp(-lamt.*Age0)); %Initial C
Cplot           =  repmat(Cloc,nr,1);            %Conc profiles
Cplot(end,:)    =  0;
C               =  Cplot(:);
t_ArK0          = -1/lamt*log(1-ke*Cplot); %Calculation of K-Ar Age
%Timeloop ------------------------------------------------------------
t_Myr = 0;
it    = 0;
while t_Myr<tlim
    it    = it+1;
    t_Myr = t_Myr+dt;
    %Time Change of Source Term & Diffusion Coefficient------------------
    Sn    =      1/ke*(lamt*exp(-lamt*(t_Myr   )));
    So    =      1/ke*(lamt*exp(-lamt*(t_Myr-dt)));
    %Integrate Source Separately =========================================
    C                =  C  +  dt*(theta*Sn+(1-theta)*So);
    C(nr:nr:nr*ndd)  =  0;
    %Diffusion Solution ==================================================
    Co    =    C(:);    %Store Old Concentration
    L_G   =      [];
    R_G   =      [];
    for idd = 1:ndd
        x_1   =    [r1d(1:end-1)]';
        x_2   =    [r1d(2:end-0)]';
        %Interp properties on path ---------------------------------
        Tn                   =       interp1(time_a(:,idd),TempK_a(:,idd),t_Myr,'linear',TminK);
        Pn                   =       interp1(time_a(:,idd),Pres_a(:,idd) ,t_Myr,'linear',Pmin);
        To                   =       interp1(time_a(:,idd),TempK_a(:,idd),t_Myr-dt,'linear',TminK);
        Po                   =       interp1(time_a(:,idd),Pres_a(:,idd) ,t_Myr-dt,'linear',Pmin);
        Trec(it,idd)         =       Tn-c2K;
        Prec(it,idd)         =       Pn;
        trec(it,idd)         =       t_Myr;
        [D0_cm2s,Ea,Va,Pref] =       diff_system(Min_Sys(idd));
        D0a2_myr             =       (D0_cm2s/(Lr(idd)*micr2cm)^2)*Myr2sec;
        Dn                   =       D0a2_myr*exp(-(Ea+(Pn-Pref)*Va)./Rgas./Tn);
        Do                   =       D0a2_myr*exp(-(Ea+(Po-Pref)*Va)./Rgas./To);
        ndim                 =       ndim_a(idd);
        D_reset(idd)         =       Dn*dt;
         if D_reset(idd)>f_loss;
             Dn = 0;
         end
        %-----------------------------------------------------------
        [L_g,R_g]            =       fill_MatrixFEM(idd,nr,ndofs,ndim,x_1,x_2,Dn,Do,theta,dt,Co);
        %Add block =======================================================
        L_G = blkdiag(L_G,L_g);
        R_G = [R_G;R_g];
    end    
    %Calculate Solution ==================================================
    if chol_sol == 1
        CH_g              = chol(L_G);        %Cholesky Factorization
        C                 = CH_g\(CH_g'\R_G); %Cholesky Solution
    else
        C                 = L_G\R_G;          %Solution
    end
    %Check Reset =========================================================
    ind_reset = zeros(ndd*nr,1);
    for idd = 1:ndd
        if D_reset(idd)>f_loss;
            C((idd-1)*nr+1:(idd-1)*nr+nr-1) = 0;
        end
    end
    %=====================================================================
    if mod(it,nout)==1 
        if flag_plot == 1
            Cplot = reshape(C,nr,ndd);
            t_ArK          = -1/lamt*log(1-ke*Cplot);  %Calculation of K-Ar Age
            %----------------------------------------------------
            subplot(121)
            plot(time_a-tlim,TempK_a-c2K,time_a-tlim,TempK_a-c2K,'LineWidth',LWW,'LineWidth',LWW)
            hold on
            plot(trec(1:nout:it,:)-tlim,Trec(1:nout:it,:),'k .','LineWidth',LWW)
            hold off
            grid on
            axis([-tlim 0.1 0 T0])
            xlabel('Age (Myr)','Interpreter','Latex','FontSize',FSS)
            ylabel('T (C)','Interpreter','Latex','FontSize',FSS)
            set(gca,'LineWidth',LWW,'FontSize',FSS)
            axis square
            
            subplot(122)
            plot(rplot,t_ArK,'LineWidth',LWW)
            grid on
            xlim([0,1.1])
            xlabel('radius [-]','Interpreter','Latex','FontSize',FSS)
            ylabel('Age (Myr)','Interpreter','Latex','FontSize',FSS)
            set(gca,'LineWidth',LWW,'FontSize',FSS)
            axis square
            title(['Max age : ',num2str(max(t_ArK(:)))])
            %---------------------------------------------------
            drawnow
        end
        if flag_echo == 1
            disp([num2str(round(it/nt*100),3),' % is done'])
        end
    end    
end
%====================================================================
time_total     = toc;
Cplot          = reshape(C,nr,ndd);
t_ArK          = -1/lamt*log(1-ke*Cplot);  %Calculation of K-Ar Age
%Parameters to save -------------------------------------------------
data_time_tot    = time_total;
data_Trec_C      = Trec;
data_trec_Myr    = trec;
data_agePath_Myr = trec-tlim;
data_C_Ar_K      = Cplot;
data_Ar_Age_Myr  = t_ArK;
data_L_microns   = Lr;
data_ndim_geom   = ndim_a;
data_Age0        = Age0;
data_Min_Sys     = Min_Sys;
data_ndd         = ndd;
data_name        = name_file;
data_r1d         = r1d;
data_nels        = nels;
data_nr          = nr;
data_El2n        = el2n;
data_El2d        = el2d;
data_ndofs       = ndofs;
data_rplot       = rplot;
data_Core_Ages   = t_ArK(1,:);
%=======================================================================
if flag_echo == 1
    disp(['Calculation is done'])
    disp(['Age (core) : ',num2str(data_Core_Ages),' Myr'])
    disp(['time for solution (sec): ',num2str(time_total)]);
end
if flag_plot == 1
    subplot(121)
    plot(time_a-tlim,TempK_a-c2K,time_a-tlim,TempK_a-c2K,'LineWidth',LWW)
    hold on
    plot(trec(1:nout:it,:)-tlim,Trec(1:nout:it,:),'k .','LineWidth',LWW)
    hold off
    grid on
    axis([-tlim 0.1 0 T0])
    xlabel('Age (Myr)','Interpreter','Latex','FontSize',FSS)
    ylabel('T (C)','Interpreter','Latex','FontSize',FSS)
    set(gca,'LineWidth',LWW,'FontSize',FSS)
    axis square
    
    subplot(122)
    plot(rplot,t_ArK,'LineWidth',LWW)
    grid on
    xlim([0,1.1])
    ylim([0 55])
    xlabel('radius [-]','Interpreter','Latex','FontSize',FSS)
    ylabel('Age (Myr)','Interpreter','Latex','FontSize',FSS)
    set(gca,'LineWidth',LWW,'FontSize',FSS)
    title(['Max age : ',num2str(max(t_ArK(:)))])
    axis square
end
if flag_save == 1
    clearvars -except data_Trec_C data_trec_Myr data_agePath_Myr data_C_Ar_K ...
              data_Ar_Age_Myr  data_L_microns data_ndim_geom data_Age0 ...
              data_Min_Sys data_ndd data_name data_r1d data_nr data_El2n data_El2d ...
              data_ndofs data_rplot data_nels data_Core_Ages
    strinG_temp = ['save data_path_',data_name]; eval(strinG_temp);    
end